function [theta_bar, phi_bar] = coarray_2D_unitary_ESPRIT( U, x_U, DD )
%COARRAY_2D_UNITARY_ESPRIT: 2D unitary ESPRIT [1] in the coarray domain
%
%	Written by Chun-Lin Liu
%	E-mail : cl.liu@caltech.edu
%	Project website:
%	https://systems.caltech.edu/dsp/students/clliu/hourglass.html
%	Last revised on July 31, 2017
%
%   If you use this script, please cite reference [1-3].
%
%	Reference
%   [1] M. D. Zoltowski, M. Haardt, and C. P. Mathews, "Closed-Form 2-D Angle Estimation with Rectangular Arrays in Element Space or Beamspace via Unitray ESPRIT, IEEE Transactions on Signal Processing, Vol. 44, No. 2, 1996
%   [2] C.-L. Liu and P. P. Vaidyanathan, “Two-Dimensional Sparse Arrays with Hole-Free Coarray and Reduced Mutual Coupling,” in Proc. of the 50th Asilomar Conference on Signals, Systems, and Computers, Pacific Grove, CA, Nov. 2016.
%   [3] C.-L. Liu and P. P. Vaidyanathan, “Hourglass Arrays, and Other Novel 2D Sparse Arrays with Reduced Mutual Coupling,” IEEE Transactions on Signal Processing, vol. 65, no. 13, pp. 3369-3383, July 2017.
%
%
%   Input:
%       1) U: The URA segment in the difference coarray [2, 3].
%       2) x_U The sample autocorrelation vector on the difference coarray
%       3) DD: The number of sources
%
%   Output:
%       1) theta_bar: normalized DOAs
%           size: DD \times 1
%       2) phi_bar: normalized DOAs
%           size: DD \times 1
%

%% ==================== The difference coarray U has to be separable ====================
[LEN_U, R] = size(U);
U_r = cell(R, 1);
LEN_U_r = zeros(1, R);
for r = 1 : R
    U_r{r} = unique(U(:, r));
    LEN_U_r(r) = length(U_r{r});
end
% Check, prod(LEN_U_r) = LEN_U
if ( prod(LEN_U_r) ~= LEN_U )
    % Coarray is not separable,
    error(message('The difference coarray is not separable.'))
end

for r = 1 : R
    if (r == 1)
        U_r_temp = U_r{1};
    else
        U_r_temp = ones(LEN_U_r(1), 1);
    end
    for rr = 2 : R
        if (rr == r)
            U_r_temp = kron(U_r_temp, U_r{r});
        else
            U_r_temp = kron(U_r_temp, ones(LEN_U_r(rr), 1));
        end
    end
    
    if (sum( U(:, r) == U_r_temp ) ~= LEN_U )
        error(message('The difference coarray D is not separable'))
    end
end

% This array is separable.
% Convert the sample autocorrelation vector x_U to the matrix X_U_mat
X_U_mat = permute( reshape( x_U, fliplr(LEN_U_r) ), R:-1:1);


%% ==================== Coarray 2D Unitary ESPRIT [1, Section V] ====================
% Before applying 2D unitary ESPRIT to X_U_mat, we need to check whether Mx
% and My are odd numbers.
[Mx, My] = size(X_U_mat);
if (mod(Mx, 2) == 0 || mod(My, 2) == 0)
    error(message('Mx and My have to be odd numbers.'));
end

Nx = ( Mx + 1 ) / 2;
Ny = ( My + 1 ) / 2;

Q_Nx = get_Q( Nx );
Q_Ny = get_Q( Ny );

% ========== Create subarrays and stack them into X \in \mathbb{R}^{NxNy \times }
X = zeros(Nx*Ny, Nx*Ny);
for ii = 0 : Nx - 1
    for jj = 0 : Ny - 1
        X(:, ii + Nx * jj + 1) = reshape( X_U_mat(ii + (0 : Nx - 1) + 1, ...
            jj + (0 : Ny - 1) + 1), [], 1 ) ;
    end
end
Y = kron(Q_Ny', Q_Nx') * X;

% ========== Step 1: Compute SVD of [Re(Y), Im(Y)] ==========
[U_mat, ~, ~] = svd( [real(Y), imag(Y)] );
% Check dimensions
if (DD > size(U_mat, 2))
    disp('Warning: DD > size(U_mat, 2), set E_S = U_mat instead')
    E_S = U_mat;
else
    E_S = U_mat(:, 1:DD);
end

% ========== Step 2a: Create P_theta ==========
QJ_2Q = get_Q( Nx-1 )' * get_J_2( Nx ) * get_Q( Nx );
K1 = real(QJ_2Q);
K2 = imag(QJ_2Q);
K_theta1 = kron( eye(Ny), K1 );
K_theta2 = kron( eye(Ny), K2 );
P_theta = (K_theta1 * E_S) \ (K_theta2 * E_S);

% ========== Step 2a: Create P_phi ==========
QJ_2Q = get_Q( Ny-1 )' * get_J_2( Ny ) * get_Q( Ny );
K3 = real(QJ_2Q);
K4 = imag(QJ_2Q);
K_phi1 = kron( K3, eye(Nx) );
K_phi2 = kron( K4, eye(Nx) );
P_phi = (K_phi1 * E_S) \ (K_phi2 * E_S);

lambda = eig(P_theta + 1i * P_phi);

theta_bar = atan(real(lambda)) / pi;
phi_bar = atan(imag(lambda)) / pi;

end

function Q_N = get_Q( N )
if ( mod(N, 2) == 0)
    K = N / 2;
    I_K = eye(K);
    Pi_K = fliplr(I_K);
    Q_N = [	I_K, 1i*I_K; ...
        Pi_K, -1i*Pi_K] / sqrt(2);
else
    K = (N - 1) / 2;
    I_K = eye(K);
    Pi_K = fliplr(I_K);
    Q_N = [	I_K, zeros(K, 1), 1i*I_K; ...
        zeros(1, K), sqrt(2), zeros(1, K); ...
        Pi_K, zeros(K, 1), -1i*Pi_K] / sqrt(2);
end
end

function J_2_N = get_J_2( N )
J_2_N = [zeros(N-1, 1), eye(N-1)];
end

