function [ D, x_D ] = x_S_to_x_D( S, x_S )
%x_S_to_x_D: convert multiple snapshots x_S(k) in the physical array domain S
% to single snapshot x_D in the difference coarray D.
%
%	Written by Chun-Lin Liu
%	E-mail : cl.liu@caltech.edu
%	Project website:
%	http://systems.caltech.edu/dsp/students/clliu/hourglass.html
%	Last revised on July 31, 2017
%
%   If you use this script, please cite reference [1] and [2].
%
%	Reference
%   [1] C.-L. Liu and P. P. Vaidyanathan, “Two-Dimensional Sparse Arrays with Hole-Free Coarray and Reduced Mutual Coupling,” in Proc. of the 50th Asilomar Conference on Signals, Systems, and Computers, Pacific Grove, CA, Nov. 2016.
%   [2] C.-L. Liu and P. P. Vaidyanathan, “Hourglass Arrays, and Other Novel 2D Sparse Arrays with Reduced Mutual Coupling,” IEEE Transactions on Signal Processing, vol. 65, no. 13, pp. 3369-3383, July 2017.
%
%
%	Input:
%       1) S: the array configuration, size: |S| \times 2.
%       2) x_S: multiple snapshot sensor measurements,
%           size: |S| \times (# of snapshots).
%
%   Output:
%       1) D: the difference coarray, size: |D| \times 2
%       2) x_D: sample autocorrelation vector 
%           size: |D| \times 1
%

% S and x_S have the same number of rows.
LEN_S = size(S, 1);
[LEN_SS, SNAPSHOTS] = size(x_S);
if (LEN_S ~= LEN_SS)
    error('Dimension mismatch.')
end

% Step 1: Evaluate the difference coarray
SS = repmat(S, [LEN_S, 1]);
SS_= kron(S, ones(LEN_S, 1));
[D, ~, ind_SS] = unique(SS - SS_, 'rows', 'sorted');
LEN_D = size(D, 1);

% Step 2 (R_S): Covariance matrix of x_S
R_S = x_S * x_S' / SNAPSHOTS;
r_S = R_S(:);

% Step 3: Sample autocorrelation vector x_D
x_D = zeros(LEN_D, 1);
for ii = 1 : LEN_D
    x_D(ii) = mean(r_S(ind_SS == ii));
end


end

